@@ -1,13 +1,15 @@
 //! calculate and collect rent from Accounts
-use solana_sdk::{
-    account::{AccountSharedData, ReadableAccount, WritableAccount},
-    clock::Epoch,
-    epoch_schedule::EpochSchedule,
-    genesis_config::GenesisConfig,
-    incinerator,
-    pubkey::Pubkey,
-    rent::{Rent, RentDue},
-    sysvar,
+use {
+    log::*,
+    solana_sdk::{
+        account::{AccountSharedData, ReadableAccount, WritableAccount},
+        clock::Epoch,
+        epoch_schedule::EpochSchedule,
+        genesis_config::GenesisConfig,
+        incinerator,
+        pubkey::Pubkey,
+        rent::{Rent, RentDue},
+    },
 };
 
 #[derive(Serialize, Deserialize, Clone, PartialEq, Debug, AbiExample)]
@@ -30,6 +32,16 @@ impl Default for RentCollector {
     }
 }
 
+/// when rent is collected for this account, this is the action to apply to the account
+#[derive(Debug)]
+pub enum RentResult {
+    /// maybe collect rent later, leave account alone
+    LeaveAloneNoRent,
+    /// collect rent
+    /// value is (new rent epoch, lamports of rent_due)
+    CollectRent((Epoch, u64)),
+}
+
 impl RentCollector {
     pub fn new(
         epoch: Epoch,
@@ -46,40 +58,69 @@ impl RentCollector {
     }
 
     pub fn clone_with_epoch(&self, epoch: Epoch) -> Self {
+        self.clone_with_epoch_and_rate(epoch, self.rent.lamports_per_byte_year)
+    }
+
+    pub fn clone_with_epoch_and_rate(&self, epoch: Epoch, lamports_per_byte_year: u64) -> Self {
+        let rent = if lamports_per_byte_year != self.rent.lamports_per_byte_year {
+            Rent {
+                lamports_per_byte_year,
+                ..self.rent
+            }
+        } else {
+            self.rent
+        };
         Self {
+            rent,
             epoch,
             ..self.clone()
         }
     }
 
     /// true if it is easy to determine this account should consider having rent collected from it
-    pub fn should_collect_rent(
-        &self,
-        address: &Pubkey,
-        account: &impl ReadableAccount,
-        rent_for_sysvars: bool,
-    ) -> bool {
+    pub fn should_collect_rent(&self, address: &Pubkey, account: &impl ReadableAccount) -> bool {
         !(account.executable() // executable accounts must be rent-exempt balance
-            || (!rent_for_sysvars && sysvar::check_id(account.owner()))
             || *address == incinerator::id())
     }
 
     /// given an account that 'should_collect_rent'
     /// returns (amount rent due, is_exempt_from_rent)
     pub fn get_rent_due(&self, account: &impl ReadableAccount) -> RentDue {
-        let slots_elapsed: u64 = (account.rent_epoch()..=self.epoch)
-            .map(|epoch| self.epoch_schedule.get_slots_in_epoch(epoch + 1))
-            .sum();
-
-        // avoid infinite rent in rust 1.45
-        let years_elapsed = if self.slots_per_year != 0.0 {
-            slots_elapsed as f64 / self.slots_per_year
+        if self
+            .rent
+            .is_exempt(account.lamports(), account.data().len())
+        {
+            RentDue::Exempt
         } else {
-            0.0
-        };
+            let account_rent_epoch = account.rent_epoch();
+            let slots_elapsed: u64 = (account_rent_epoch..=self.epoch)
+                .map(|epoch| self.epoch_schedule.get_slots_in_epoch(epoch + 1))
+                .sum();
 
-        self.rent
-            .due(account.lamports(), account.data().len(), years_elapsed)
+            // avoid infinite rent in rust 1.45
+            let years_elapsed = if self.slots_per_year != 0.0 {
+                slots_elapsed as f64 / self.slots_per_year
+            } else {
+                0.0
+            };
+
+            // we know this account is not exempt
+            let due = self.rent.due_amount(account.data().len(), years_elapsed);
+
+            // we expect rent_epoch to always be one of: {0, self.epoch-1, self.epoch, self.epoch+1}
+            if account_rent_epoch != 0
+                && (account_rent_epoch + 1 < self.epoch || account_rent_epoch > self.epoch + 1)
+            {
+                // this should not occur in a running validator
+                if due == 0 {
+                    inc_new_counter_info!("rent-collector-rent-epoch-range-large-exempt", 1);
+                } else {
+                    inc_new_counter_info!("rent-collector-rent-epoch-range-large-paying", 1);
+                }
+            }
+
+            RentDue::Paying(due)
+        }
     }
 
     // Updates the account's lamports and status, and returns the amount of rent collected, if any.
@@ -90,18 +131,45 @@ impl RentCollector {
         &self,
         address: &Pubkey,
         account: &mut AccountSharedData,
-        rent_for_sysvars: bool,
         filler_account_suffix: Option<&Pubkey>,
     ) -> CollectedInfo {
-        if self.can_skip_rent_collection(address, account, rent_for_sysvars, filler_account_suffix)
-        {
-            return CollectedInfo::default();
+        match self.calculate_rent_result(address, account, filler_account_suffix) {
+            RentResult::LeaveAloneNoRent => CollectedInfo::default(),
+            RentResult::CollectRent((next_epoch, rent_due)) => {
+                account.set_rent_epoch(next_epoch);
+
+                let begin_lamports = account.lamports();
+                account.saturating_sub_lamports(rent_due);
+                let end_lamports = account.lamports();
+                let mut account_data_len_reclaimed = 0;
+                if end_lamports == 0 {
+                    account_data_len_reclaimed = account.data().len() as u64;
+                    *account = AccountSharedData::default();
+                }
+                CollectedInfo {
+                    rent_amount: begin_lamports - end_lamports,
+                    account_data_len_reclaimed,
+                }
+            }
+        }
+    }
+
+    /// determine what should happen to collect rent from this account
+    #[must_use]
+    pub fn calculate_rent_result(
+        &self,
+        address: &Pubkey,
+        account: &impl ReadableAccount,
+        filler_account_suffix: Option<&Pubkey>,
+    ) -> RentResult {
+        if self.can_skip_rent_collection(address, account, filler_account_suffix) {
+            return RentResult::LeaveAloneNoRent;
         }
 
         let rent_due = self.get_rent_due(account);
         if let RentDue::Paying(0) = rent_due {
             // maybe collect rent later, leave account alone
-            return CollectedInfo::default();
+            return RentResult::LeaveAloneNoRent;
         }
 
         let epoch_increment = match rent_due {
@@ -111,22 +179,7 @@ impl RentCollector {
             // Rent is collected for next epoch
             RentDue::Paying(_) => 1,
         };
-        account.set_rent_epoch(self.epoch + epoch_increment);
-
-        let begin_lamports = account.lamports();
-        account.saturating_sub_lamports(rent_due.lamports());
-        let end_lamports = account.lamports();
-
-        let mut account_data_len_reclaimed = 0;
-        if end_lamports == 0 {
-            account_data_len_reclaimed = account.data().len() as u64;
-            *account = AccountSharedData::default();
-        }
-
-        CollectedInfo {
-            rent_amount: begin_lamports - end_lamports,
-            account_data_len_reclaimed,
-        }
+        RentResult::CollectRent((self.epoch + epoch_increment, rent_due.lamports()))
     }
 
     #[must_use = "add to Bank::collected_rent"]
@@ -134,22 +187,20 @@ impl RentCollector {
         &self,
         address: &Pubkey,
         account: &mut AccountSharedData,
-        rent_for_sysvars: bool,
     ) -> CollectedInfo {
         // initialize rent_epoch as created at this epoch
         account.set_rent_epoch(self.epoch);
-        self.collect_from_existing_account(address, account, rent_for_sysvars, None)
+        self.collect_from_existing_account(address, account, None)
     }
 
     /// Performs easy checks to see if rent collection can be skipped
     fn can_skip_rent_collection(
         &self,
         address: &Pubkey,
-        account: &mut AccountSharedData,
-        rent_for_sysvars: bool,
+        account: &impl ReadableAccount,
         filler_account_suffix: Option<&Pubkey>,
     ) -> bool {
-        !self.should_collect_rent(address, account, rent_for_sysvars)
+        !self.should_collect_rent(address, account)
             || account.rent_epoch() > self.epoch
             || crate::accounts_db::AccountsDb::is_filler_account_helper(
                 address,
@@ -186,13 +237,20 @@ impl std::ops::AddAssign for CollectedInfo {
 
 #[cfg(test)]
 mod tests {
-    use {super::*, solana_sdk::account::Account};
+    use {
+        super::*,
+        solana_sdk::{account::Account, sysvar},
+    };
+
+    fn default_rent_collector_clone_with_epoch(epoch: Epoch) -> RentCollector {
+        RentCollector::default().clone_with_epoch(epoch)
+    }
 
     #[test]
     fn test_collect_from_account_created_and_existing() {
         let old_lamports = 1000;
         let old_epoch = 1;
-        let new_epoch = 3;
+        let new_epoch = 2;
 
         let (mut created_account, mut existing_account) = {
             let account = AccountSharedData::from(Account {
@@ -204,14 +262,11 @@ mod tests {
             (account.clone(), account)
         };
 
-        let rent_collector = RentCollector::default().clone_with_epoch(new_epoch);
+        let rent_collector = default_rent_collector_clone_with_epoch(new_epoch);
 
         // collect rent on a newly-created account
-        let collected = rent_collector.collect_from_created_account(
-            &solana_sdk::pubkey::new_rand(),
-            &mut created_account,
-            true,
-        );
+        let collected = rent_collector
+            .collect_from_created_account(&solana_sdk::pubkey::new_rand(), &mut created_account);
         assert!(created_account.lamports() < old_lamports);
         assert_eq!(
             created_account.lamports() + collected.rent_amount,
@@ -224,7 +279,6 @@ mod tests {
         let collected = rent_collector.collect_from_existing_account(
             &solana_sdk::pubkey::new_rand(),
             &mut existing_account,
-            true,
             None,
         );
         assert!(existing_account.lamports() < old_lamports);
@@ -252,11 +306,10 @@ mod tests {
         assert_eq!(account.rent_epoch(), 0);
 
         // create a tested rent collector
-        let rent_collector = RentCollector::default().clone_with_epoch(epoch);
+        let rent_collector = default_rent_collector_clone_with_epoch(epoch);
 
         // first mark account as being collected while being rent-exempt
-        let collected =
-            rent_collector.collect_from_existing_account(&pubkey, &mut account, true, None);
+        let collected = rent_collector.collect_from_existing_account(&pubkey, &mut account, None);
         assert_eq!(account.lamports(), huge_lamports);
         assert_eq!(collected, CollectedInfo::default());
 
@@ -264,8 +317,7 @@ mod tests {
         account.set_lamports(tiny_lamports);
 
         // ... and trigger another rent collection on the same epoch and check that rent is working
-        let collected =
-            rent_collector.collect_from_existing_account(&pubkey, &mut account, true, None);
+        let collected = rent_collector.collect_from_existing_account(&pubkey, &mut account, None);
         assert_eq!(account.lamports(), tiny_lamports - collected.rent_amount);
         assert_ne!(collected, CollectedInfo::default());
     }
@@ -282,17 +334,9 @@ mod tests {
         assert_eq!(account.rent_epoch(), 0);
 
         let epoch = 3;
-        let rent_collector = RentCollector::default().clone_with_epoch(epoch);
+        let rent_collector = default_rent_collector_clone_with_epoch(epoch);
 
-        // old behavior: sysvars are special-cased
-        let collected =
-            rent_collector.collect_from_existing_account(&pubkey, &mut account, false, None);
-        assert_eq!(account.lamports(), tiny_lamports);
-        assert_eq!(collected, CollectedInfo::default());
-
-        // new behavior: sysvars are NOT special-cased
-        let collected =
-            rent_collector.collect_from_existing_account(&pubkey, &mut account, true, None);
+        let collected = rent_collector.collect_from_existing_account(&pubkey, &mut account, None);
         assert_eq!(account.lamports(), 0);
         assert_eq!(collected.rent_amount, 1);
     }
@@ -310,14 +354,10 @@ mod tests {
             rent_epoch: account_rent_epoch,
             ..Account::default()
         });
-        let rent_collector = RentCollector::default().clone_with_epoch(account_rent_epoch + 2);
+        let rent_collector = default_rent_collector_clone_with_epoch(account_rent_epoch + 1);
 
-        let collected = rent_collector.collect_from_existing_account(
-            &Pubkey::new_unique(),
-            &mut account,
-            true,
-            None,
-        );
+        let collected =
+            rent_collector.collect_from_existing_account(&Pubkey::new_unique(), &mut account, None);
 
         assert_eq!(collected.rent_amount, account_lamports);
         assert_eq!(
